@@ -1,11 +1,16 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,78 +19,61 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
- * $Id: display.h,v 1.9 1994/05/31 12:31:54 mlschroe Exp $ FAU
+ * $Id$ GNU
  */
 
-#ifdef MAPKEYS
-struct kmap
-{
-  char seq[8];
-  char off[8];
-  int  nr;
-};
+#ifndef SCREEN_DISPLAY_H
+#define SCREEN_DISPLAY_H
 
-#define KMAP_SEQ ((int)((struct kmap *)0)->seq)
-#define KMAP_OFF ((int)((struct kmap *)0)->off)
+#include "layout.h"
+#include "canvas.h"
+#include "viewport.h"
+
+#ifdef MAPKEYS
 
 #define KMAP_KEYS (T_OCAPS-T_CAPS)
 #define KMAP_AKEYS (T_OCAPS-T_CURSOR)
-#define KMAP_EXT 50
 
 #define KMAP_NOTIMEOUT 0x4000
 
+struct kmap_ext
+{
+  char *str;
+  int fl;
+  struct action um;
+  struct action dm;
+  struct action mm;
+};
+
+#else
+
+#define KMAP_KEYS 0
+
 #endif
 
 struct win;			/* forward declaration */
 
-struct canvas
-{
-  struct canvas   *c_next;	/* next canvas on display */
-  struct display  *c_display;	/* back pointer to display */
-  struct viewport *c_vplist;
-  struct layer    *c_layer;	/* layer on this canvas */
-  struct canvas   *c_lnext;	/* next canvas that displays layer */
-  struct layer     c_blank;	/* bottom layer, always blank */
-  int              c_xoff;	/* canvas x offset on display */
-  int              c_yoff;	/* canvas y offset on display */
-  int              c_xs;
-  int              c_xe;
-  int              c_ys;
-  int              c_ye;
-  struct event     c_captev;	/* caption changed event */
-};
-
-struct viewport
-{
-  struct viewport *v_next;	/* next vp on canvas */
-  struct canvas   *v_canvas;	/* back pointer to canvas */
-  int              v_xoff;	/* layer x offset on display */
-  int              v_yoff;	/* layer y offset on display */
-  int              v_xs;	/* vp upper left */
-  int              v_xe;	/* vp upper right */
-  int              v_ys;	/* vp lower left */
-  int              v_ye;	/* vp lower right */
-};
-
 struct display
 {
   struct display *d_next;	/* linked list */
-  struct acluser *d_user;		/* user who owns that display */
+  struct acluser *d_user;	/* user who owns that display */
+  struct canvas d_canvas;	/* our canvas slice */
   struct canvas *d_cvlist;	/* the canvases of this display */
   struct canvas *d_forecv;	/* current input focus */
+  struct layout *d_layout;	/* layout we're using */
   void (*d_processinput) __P((char *, int));
   char *d_processinputdata;	/* data for processinput */
   int d_vpxmin, d_vpxmax;	/* min/max used position on display */
   struct win *d_fore;		/* pointer to fore window */
   struct win *d_other;		/* pointer to other window */
-  char  d_nonblock;		/* 1: don't block if obufmax reached */
-				/* 2: obufmax is reached, discard */
-  char  d_termname[20 + 1];	/* $TERM */
+  int   d_nonblock;		/* -1 don't block if obufmax reached */
+				/* >0: block after nonblock secs */
+  char  d_termname[MAXTERMLEN + 1];	/* $TERM */
   char	*d_tentry;		/* buffer for tgetstr */
   char	d_tcinited;		/* termcap inited flag */
   int	d_width, d_height;	/* width/height of the screen */
@@ -112,11 +100,13 @@ struct display
   int	d_hstatus;		/* hardstatus used */
   int	d_lp_missing;		/* last character on bot line missing */
   int   d_mouse;		/* mouse mode */
+  int	d_mousetrack;		/* set when user wants to use mouse even when the window
+				   does not */
 #ifdef RXVT_OSC
   int   d_xtermosc[4];		/* osc used */
 #endif
   struct mchar d_lpchar;	/* missing char */
-  time_t d_status_time;		/* time of status display */
+  struct timeval d_status_time;	/* time of status display */
   int   d_status;		/* is status displayed? */
   char	d_status_bell;		/* is it only a vbell? */
   int	d_status_len;		/* length of status line */
@@ -126,6 +116,7 @@ struct display
   int	d_status_lasty;		/*   before status was displayed */
   int   d_status_obuflen;	/* saved obuflen */ 
   int   d_status_obuffree;	/* saved obuffree */ 
+  int	d_status_obufpos;	/* end of status position in obuf */
   struct event d_statusev;	/* timeout event */
   struct event d_hstatusev;	/* hstatus changed event */
   int	d_kaablamm;		/* display kaablamm msg */
@@ -135,6 +126,7 @@ struct display
   int	d_userfd;		/* fd of the tty */
   struct event d_readev;	/* userfd read event */
   struct event d_writeev;	/* userfd write event */
+  struct event d_blockedev;	/* blocked timeout */
   struct mode d_OldMode;	/* tty mode when screen was started */
   struct mode d_NewMode;	/* New tty mode */
   int	d_flow;			/* tty's flow control on/off flag*/
@@ -150,12 +142,14 @@ struct display
 #endif
 #ifdef MAPKEYS
   int	d_nseqs;		/* number of valid mappings */
-  char *d_seqp;			/* pointer into keymap array */
+  int	d_aseqs;		/* number of allocated mappings */
+  unsigned char  *d_kmaps;	/* keymaps */
+  unsigned char *d_seqp;	/* pointer into keymap array */
   int	d_seql;			/* number of parsed chars */
+  unsigned char *d_seqh;	/* last hit */
   struct event d_mapev;		/* timeout event */
   int	d_dontmap;		/* do not map next */
   int	d_mapdefault;		/* do map next to default */
-  struct kmap d_kmaps[KMAP_KEYS+KMAP_EXT];	/* keymaps */
 #endif
   union	tcu d_tcs[T_N];		/* terminal capabilities */
   char *d_attrtab[NATTR];	/* attrib emulation table */
@@ -176,6 +170,13 @@ struct display
 # ifdef _SEQUENT_
   char	d_loginhost[100+1];
 # endif /* _SEQUENT_ */
+#endif
+  int   d_blocked;
+  int   d_blocked_fuzz;
+  struct event d_idleev;	/* screen blanker */
+#ifdef BLANKER_PRG
+  int   d_blankerpid;
+  struct event d_blankerev;
 #endif
 };
 
@@ -188,7 +189,9 @@ extern struct display TheDisplay;
 
 #define D_user		DISPLAY(d_user)
 #define D_username	(DISPLAY(d_user) ? DISPLAY(d_user)->u_name : 0)
+#define D_canvas	DISPLAY(d_canvas)
 #define D_cvlist	DISPLAY(d_cvlist)
+#define D_layout	DISPLAY(d_layout)
 #define D_forecv	DISPLAY(d_forecv)
 #define D_processinput	DISPLAY(d_processinput)
 #define D_processinputdata	DISPLAY(d_processinputdata)
@@ -224,6 +227,7 @@ extern struct display TheDisplay;
 #define D_hstatus	DISPLAY(d_hstatus)
 #define D_lp_missing	DISPLAY(d_lp_missing)
 #define D_mouse		DISPLAY(d_mouse)
+#define D_mousetrack	DISPLAY(d_mousetrack)
 #define D_xtermosc	DISPLAY(d_xtermosc)
 #define D_lpchar	DISPLAY(d_lpchar)
 #define D_status	DISPLAY(d_status)
@@ -236,6 +240,7 @@ extern struct display TheDisplay;
 #define D_status_lasty	DISPLAY(d_status_lasty)
 #define D_status_obuflen	DISPLAY(d_status_obuflen)
 #define D_status_obuffree	DISPLAY(d_status_obuffree)
+#define D_status_obufpos	DISPLAY(d_status_obufpos)
 #define D_statusev	DISPLAY(d_statusev)
 #define D_hstatusev	DISPLAY(d_hstatusev)
 #define D_kaablamm	DISPLAY(d_kaablamm)
@@ -255,8 +260,10 @@ extern struct display TheDisplay;
 #define D_obuffree	DISPLAY(d_obuffree)
 #define D_auto_nuke	DISPLAY(d_auto_nuke)
 #define D_nseqs		DISPLAY(d_nseqs)
+#define D_aseqs		DISPLAY(d_aseqs)
 #define D_seqp		DISPLAY(d_seqp)
 #define D_seql		DISPLAY(d_seql)
+#define D_seqh		DISPLAY(d_seqh)
 #define D_dontmap	DISPLAY(d_dontmap)
 #define D_mapdefault	DISPLAY(d_mapdefault)
 #define D_kmaps		DISPLAY(d_kmaps)
@@ -282,7 +289,13 @@ extern struct display TheDisplay;
 #define D_loginhost	DISPLAY(d_loginhost)
 #define D_readev	DISPLAY(d_readev)
 #define D_writeev	DISPLAY(d_writeev)
+#define D_blockedev	DISPLAY(d_blockedev)
 #define D_mapev		DISPLAY(d_mapev)
+#define D_blocked	DISPLAY(d_blocked)
+#define D_blocked_fuzz	DISPLAY(d_blocked_fuzz)
+#define D_idleev	DISPLAY(d_idleev)
+#define D_blankerev	DISPLAY(d_blankerev)
+#define D_blankerpid	DISPLAY(d_blankerpid)
 
 
 #define GRAIN 4096	/* Allocation grain size for output buffer */
@@ -299,23 +312,6 @@ do				\
   }				\
 while (0)
 
-#define CV_CALL(cv, cmd)			\
-{						\
-  struct display *olddisplay = display;		\
-  struct layer *oldflayer = flayer;		\
-  struct layer *l = cv->c_layer;		\
-  struct canvas *cvlist = l->l_cvlist;		\
-  struct canvas *cvlnext = cv->c_lnext;		\
-  flayer = l;					\
-  l->l_cvlist = cv;				\
-  cv->c_lnext = 0;				\
-  cmd;						\
-  flayer = oldflayer;				\
-  l->l_cvlist = cvlist;				\
-  cv->c_lnext = cvlnext;			\
-  display = olddisplay;				\
-}
-
 #define STATUS_OFF	0
 #define STATUS_ON_WIN	1
 #define STATUS_ON_HS	2
@@ -325,3 +321,6 @@ while (0)
 #define HSTATUS_MESSAGE		2
 #define HSTATUS_HS		3
 #define HSTATUS_ALWAYS		(1<<2)
+
+#endif /* SCREEN_DISPLAY_H */
+
