@@ -1,11 +1,19 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2010
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,16 +22,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
  */
 
-#include "rcs.h"
-RCS_ID("$Id: help.c,v 1.7 1994/05/31 12:32:04 mlschroe Exp $ FAU")
-
 #include <sys/types.h>
 
 #include "config.h"
@@ -31,14 +36,24 @@ RCS_ID("$Id: help.c,v 1.7 1994/05/31 12:32:04 mlschroe Exp $ FAU")
 #include "screen.h"
 #include "extern.h"
 
-char version[40];      /* initialised by main() */
+#include "list_generic.h"
+
+char version[60];      /* initialised by main() */
 
 extern struct layer *flayer;
 extern struct display *display, *displays;
+extern struct win *windows;
+extern int maxwin;
 extern char *noargs[];
 extern struct mchar mchar_blank, mchar_so;
-extern char *blank;
-extern struct win *wtab[];
+extern int renditions[];
+extern unsigned char *blank;
+extern struct win **wtab;
+#ifdef MAPKEYS
+extern struct term term[];
+#endif
+
+extern struct LayFuncs ListLf;
 
 static void PadStr __P((char *, int, int, int));
 
@@ -67,13 +82,15 @@ char *myname, *message, *arg;
 #if defined(LOGOUTOK) && defined(UTMPOK)
   printf("-l            Login mode on (update %s), -ln = off.\n", UTMPFILE);
 #endif
-  printf("-list         or -ls. Do nothing, just list our SockDir.\n");
+  printf("-ls [match]   or\n");
+  printf("-list         Do nothing, just list our SockDir [on possible matches].\n");
   printf("-L            Turn on output logging.\n");
   printf("-m            ignore $STY variable, do create a new screen session.\n");
   printf("-O            Choose optimal output rather than exact vt100 emulation.\n");
   printf("-p window     Preselect the named window if it exists.\n");
   printf("-q            Quiet startup. Exits with non-zero return code if unsuccessful.\n");
-  printf("-r            Reattach to a detached screen process.\n");
+  printf("-Q            Commands will send the response to the stdout of the querying process.\n");
+  printf("-r [session]  Reattach to a detached screen process.\n");
   printf("-R            Reattach if possible, otherwise start a new session.\n");
   printf("-s shell      Shell to execute rather than $SHELL.\n");
   printf("-S sockname   Name this session <pid>.sockname instead of <pid>.<tty>.<host>.\n");
@@ -83,7 +100,7 @@ char *myname, *message, *arg;
   printf("-U            Tell screen to use UTF-8 encoding.\n");
 #endif
   printf("-v            Print \"Screen version %s\".\n", version);
-  printf("-wipe         Do nothing, just clean up SockDir.\n");
+  printf("-wipe [match] Do nothing, just clean up SockDir [on possible matches].\n");
 #ifdef MULTI
   printf("-x            Attach to a not detached screen. (Multi display mode).\n");
 #endif /* MULTI */
@@ -133,7 +150,8 @@ static struct LayFuncs HelpLf =
   DefClearLine,
   DefRewrite,
   DefResize,
-  DefRestore
+  DefRestore,
+  0
 };
 
 
@@ -163,7 +181,7 @@ struct action *ktabp;
     used[n] = 0;
   mcom = 0;
   mkey = 0;
-  for (key = 0; key < 256; key++)
+  for (key = 0; key < 256 + KMAP_KEYS; key++)
     {
       n = ktabp[key].nr;
       if (n == RC_ILLEGAL)
@@ -316,7 +334,7 @@ helppage()
 	      x += helpdata->inter - !col;
 	      n = helpdata->nact[n];
 	      buf[0] = '\0';
-	      for (key = 0; key < 256; key++)
+	      for (key = 0; key < 256 + KMAP_KEYS; key++)
 		if (ktabp[key].nr == n && ktabp[key].args == noargs && strlen(buf) < sizeof(buf) - 7)
 		  {
 		    strcat(buf, " ");
@@ -335,13 +353,13 @@ helppage()
 	  while ((n = ktabp[helpdata->command_search].nr) == RC_ILLEGAL
 		 || ktabp[helpdata->command_search].args == noargs)
 	    {
-	      if (++helpdata->command_search >= 256)
+	      if (++helpdata->command_search >= 256 + KMAP_KEYS)
 		return -1;
 	    }
 	  buf[0] = '\0';
 	  add_key_to_buf(buf, helpdata->command_search);
-	  PadStr(buf, 4, 0, crow);
-	  AddAction(&ktabp[helpdata->command_search++], 4, crow);
+	  PadStr(buf, 5, 0, crow);
+	  AddAction(&ktabp[helpdata->command_search++], 5, crow);
 	  helpdata->grow++;
 	}
       else
@@ -362,6 +380,7 @@ int x, y;
   char buf[256];
   int del, l;
   char *bp, *cp, **pp;
+  int *lp, ll;
   int fr;
   struct mchar mchar_dol;
 
@@ -381,18 +400,20 @@ int x, y;
   LPutChar(flayer, fr ? &mchar_blank : &mchar_dol, x++, y);
 
   pp = act->args;
+  lp = act->argl;
   while (pp && (cp = *pp) != NULL)
     {
       del = 0;
       bp = buf;
-      if (!*cp || (index(cp, ' ') != NULL))
+      ll = *lp++;
+      if (!ll || (index(cp, ' ') != NULL))
 	{
 	  if (index(cp, '\'') != NULL)
 	    *bp++ = del = '"';
 	  else
 	    *bp++ = del = '\'';
 	}
-      while (*cp && bp < buf + 250)
+      while (ll-- && bp < buf + 250)
 	bp += AddXChar(bp, *(unsigned char *)cp++);
       if (del)
 	*bp++ = del;
@@ -424,6 +445,17 @@ int key;
     strcpy(buf, "unset");
   else if (key == ' ')
     strcpy(buf, "sp");
+#ifdef MAPKEYS
+  else if (key >= 256)
+    {
+      key = key - 256 + T_CAPS;
+      buf[0] = ':';
+      buf[1] = term[key].tcname[0];
+      buf[2] = term[key].tcname[1];
+      buf[3] = ':';
+      buf[4] = 0;
+    }
+#endif
   else
     buf[AddXChar(buf, key)] = 0;
 }
@@ -475,19 +507,22 @@ static struct LayFuncs CopyrightLf =
   DefClearLine,
   DefRewrite,
   DefResize,
-  DefRestore
+  DefRestore,
+  0
 };
 
 static const char cpmsg[] = "\
 \n\
 Screen version %v\n\
 \n\
-Copyright (c) 1993-2002 Juergen Weigert, Michael Schroeder\n\
+Copyright (c) 2010 Juergen Weigert, Sadrul Habib Chowdhury\n\
+Copyright (c) 2008, 2009 Juergen Weigert, Michael Schroeder, Micah Cowan, Sadrul Habib Chowdhury\n\
+Copyright (c) 1993-2002, 2003, 2005, 2006, 2007 Juergen Weigert, Michael Schroeder\n\
 Copyright (c) 1987 Oliver Laumann\n\
 \n\
 This program is free software; you can redistribute it and/or \
 modify it under the terms of the GNU General Public License as published \
-by the Free Software Foundation; either version 2, or (at your option) \
+by the Free Software Foundation; either version 3, or (at your option) \
 any later version.\n\
 \n\
 This program is distributed in the hope that it will be useful, \
@@ -496,13 +531,79 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the \
 GNU General Public License for more details.\n\
 \n\
 You should have received a copy of the GNU General Public License \
-along with this program (see the file COPYING); if not, write to the \
-Free Software Foundation, Inc., \
-59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n\
+along with this program (see the file COPYING); if not, see \
+http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc., \
+51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA.\n\
 \n\
 Send bugreports, fixes, enhancements, t-shirts, money, beer & pizza to \
-screen@uni-erlangen.de\n";
+screen-devel@gnu.org\n\n\n"
 
+"Capabilities:\n"
+
+#ifdef COPY_PASTE
+"+copy "
+#else
+"-copy "
+#endif
+
+#ifdef REMOTE_DETACH
+"+remote-detach "
+#else
+"-remote-detach "
+#endif
+
+#ifdef POW_DETACH
+"+power-detach "
+#else
+"-power-detach "
+#endif
+
+#ifdef MULTI
+"+multi-attach "
+#else
+"-multi-attach "
+#endif
+
+#ifdef MULTIUSER
+"+multi-user "
+#else
+"-multi-user "
+#endif
+
+#ifdef FONT
+"+font "
+#else
+"-font "
+#endif
+
+#ifdef COLORS256
+"+color-256 "
+#elif defined(COLORS16)
+"+color-16 "
+#elif defined(COLOR)
+"+color "
+#else
+"-color "
+#endif
+
+#ifdef UTF8
+"+utf8 "
+#else
+"-utf8 "
+#endif
+
+#ifdef RXVT_OSC
+"+rxvt "
+#else
+"-rxvt "
+#endif
+
+#ifdef BUILTIN_TELNET
+"+builtin-telnet "
+#else
+"-builtin-telnet "
+#endif
+;
 
 static void
 CopyrightProcess(ppbuf, plen)
@@ -658,623 +759,6 @@ int y, xs, xe, isblank;
 }
 
 
-
-/*
-**
-**    here is all the displays stuff 
-**
-*/
-
-#ifdef MULTI
-
-static void DisplaysProcess __P((char **, int *));
-static void DisplaysRedisplayLine __P((int, int, int, int));
-static void displayspage __P((void));
-
-struct displaysdata
-{
-  int dummy_element_for_solaris;
-};
-
-static struct LayFuncs DisplaysLf =
-{
-  DisplaysProcess,
-  HelpAbort,
-  DisplaysRedisplayLine,
-  DefClearLine,
-  DefRewrite,
-  DefResize,
-  DefRestore
-};
-
-static void
-DisplaysProcess(ppbuf, plen)
-char **ppbuf;
-int *plen;
-{
-  int done = 0;
-
-  ASSERT(flayer);
-  while (!done && *plen > 0)
-    {
-      switch (**ppbuf)
-	{
-	case ' ':
-	  displayspage();
-	  break;
-	case '\r':
-	case '\n':
-	  HelpAbort();
-	  done = 1;
-	  break;
-	default:
-	  break;
-	}
-      ++*ppbuf;
-      --*plen;
-    }
-}
-
-
-void
-display_displays()
-{
-  if (flayer->l_width < 10 || flayer->l_height < 5)
-    {
-      LMsg(0, "Window size too small for displays page");
-      return;
-    }
-  if (InitOverlayPage(sizeof(struct displaysdata), &DisplaysLf, 0))
-    return;
-  flayer->l_x = 0;
-  flayer->l_y = flayer->l_height - 1;
-  displayspage();
-}
-
-/*
- * layout of the displays page is as follows:
-
-xterm 80x42      jnweiger@/dev/ttyp4    0(m11)    &rWx
-facit 80x24 nb   mlschroe@/dev/ttyhf   11(tcsh)    rwx
-xterm 80x42      jnhollma@/dev/ttyp5    0(m11)    &R.x
-
-  |     |    |      |         |         |   |     | ___ window permissions 
-  |     |    |      |         |         |   |     |      (R. is locked r-only,
-  |     |    |      |         |         |   |     |       W has wlock)
-  |     |    |      |         |         |   |     |___ Window is shared
-  |     |    |      |         |         |   |___ Name/Title of window
-  |     |    |      |         |         |___ Number of window
-  |     |    |      |         |___ Name of the display (the attached device)
-  |     |    |      |___ Username who is logged in at the display
-  |     |    |___ Display is in nonblocking mode. Shows 'NB' if obuf is full.
-  |     |___ Displays geometry as width x height.
-  |___ the terminal type known by screen for this display.
- 
- */
-
-static void
-displayspage()
-{
-  int y, l;
-  char tbuf[80];
-  struct display *d;
-  struct win *w;
-
-  LClearAll(flayer, 0);
-
-  leftline("term-type   size         user interface           window", 0);  
-  leftline("---------- ------- ---------- ----------------- ----------", 1);
-  y = 2;
-  
-  for (d = displays; d; d = d->d_next)
-    {
-      w = d->d_fore;
-
-      if (y >= flayer->l_height - 3)
-	break;
-      sprintf(tbuf, "%-10.10s%4dx%-4d%10.10s@%-16.16s%s",
-	      d->d_termname, d->d_width, d->d_height, d->d_user->u_name, 
-	      d->d_usertty, 
-	      d->d_nonblock ? ((( d->d_obufp - d->d_obuf) > d->d_obufmax) ?
-		"NB" : "nb") : "  ");
-
-      if (w)
-	{
-	  l = 10 - strlen(w->w_title);
-	  if (l < 0) 
-	    l = 0;
-	  sprintf(tbuf + strlen(tbuf), "%3d(%.10s)%*s%c%c%c%c",
-		  w->w_number, w->w_title, l, "", 
-		  /* w->w_dlist->next */ 0 ? '&' : ' ', 
-		  /*
-		   * The rwx triple:
-		   * -,r,R	no read, read, read only due to foreign wlock
-		   * -,.,w,W	no write, write suppressed by foreign wlock,
-		   *            write, own wlock
-		   * -,x	no execute, execute
-		   */
-#ifdef MULTIUSER
-		  (AclCheckPermWin(d->d_user, ACL_READ, w) ? '-' : 
-		   ((w->w_wlock == WLOCK_OFF || d->d_user == w->w_wlockuser) ?
-		    'r' : 'R')),
-		  (AclCheckPermWin(d->d_user, ACL_READ, w) ? '-' :
-		   ((w->w_wlock == WLOCK_OFF) ? 'w' :
-		    ((d->d_user == w->w_wlockuser) ? 'W' : 'v'))),
-		  (AclCheckPermWin(d->d_user, ACL_READ, w) ? '-' : 'x')
-#else
-		  'r', 'w', 'x'
-#endif
-	  );
-	}
-      leftline(tbuf, y);
-      y++;
-    }
-  sprintf(tbuf,"[Press Space %s Return to end.]",
-	  1 ? "to refresh;" : "or");
-  centerline(tbuf, flayer->l_height - 2);
-  LaySetCursor();
-}
-
-static void
-DisplaysRedisplayLine(y, xs, xe, isblank)
-int y, xs, xe, isblank;
-{
-  ASSERT(flayer);
-  if (y < 0)
-    {
-      displayspage();
-      return;
-    }
-  if (y != 0 && y != flayer->l_height - 1)
-    return;
-  if (isblank)
-    return;
-  LClearArea(flayer, xs, y, xe, y, 0, 0);
-  /* To be filled in... */
-}
-
-#endif /* MULTI */
-
-
-/*
-**
-**    here is the windowlist
-**
-*/
-
-static void WListProcess __P((char **, int *));
-static void WListRedisplayLine __P((int, int, int, int));
-static void wlistpage __P((void));
-static void WListLine __P((int, int, int, int));
-static void WListLines __P((int, int));
-static void WListMove __P((int, int));
-static void WListUpdate __P((struct win *));
-static int  WListNormalize __P((void));
-static int  WListResize __P((int, int));
-
-struct wlistdata {
-  int pos;
-  int ypos;
-  int npos;
-  int numwin;
-  int first;
-  int last;
-  int start;
-};
-
-static struct LayFuncs WListLf =
-{
-  WListProcess,
-  HelpAbort,
-  WListRedisplayLine,
-  DefClearLine,
-  DefRewrite,
-  WListResize,
-  DefRestore
-};
-
-static int
-WListResize(wi, he)
-int wi, he;
-{
-  struct wlistdata *wlistdata;
-  if (wi < 10 || he < 5)
-    return -1;
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  flayer->l_width = wi;
-  flayer->l_height = he;
-  wlistdata->numwin = he - 3;
-  if (wlistdata->ypos >= wlistdata->numwin)
-    wlistdata->ypos = wlistdata->numwin - 1;
-  flayer->l_y = he - 1;
-  return 0;
-}
-
-static void
-WListProcess(ppbuf, plen)
-char **ppbuf;
-int *plen;
-{
-  int done = 0;
-  struct wlistdata *wlistdata;
-  int h;
-
-  ASSERT(flayer);
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  h = wlistdata->numwin;
-  while (!done && *plen > 0)
-    {
-      if ((unsigned char)**ppbuf >= '0' && (unsigned char)**ppbuf <= '9')
-	{
-	  int n = (unsigned char)**ppbuf - '0';
-	  int d = 0;
-	  if (n < MAXWIN && wtab[n])
-	    {
-	      while (wlistdata->pos > n)
-		{
-		  if (wtab[n])
-		    d--;
-		  n++;
-		}
-	      while (wlistdata->pos < n)
-		{
-		  if (wtab[n])
-		    d++;
-		  n--;
-		}
-	    }
-	  if (d)
-	    WListMove(d, -1);
-	}
-      switch ((unsigned char)**ppbuf)
-	{
-	case 0220:	/* up */
-	case 16:	/* ^P like emacs */
-	case 'k':
-	  WListMove(-1, -1);
-	  break;
-	case 0216:	/* down */
-	case 14:	/* ^N like emacs */
-	case 'j':
-	  WListMove(1, -1);
-	  break;
-	case '\025':
-	  WListMove(-(h / 2), wlistdata->ypos);
-	  break;
-	case '\004':
-	  WListMove(h / 2, wlistdata->ypos);
-	  break;
-	case 0002:
-	case 'b':
-	  WListMove(-h, -1);
-	  break;
-	case 0006:
-	case 'f':
-	  WListMove(h, -1);
-	  break;
-	case 0201:	/* home */
-	  WListMove(-wlistdata->pos, -1);
-	  break;
-	case 0205:	/* end */
-	  WListMove(MAXWIN, -1);
-	  break;
-	case '\r':
-	case '\n':
-	case ' ':
-	  done = 1;
-	  h = wlistdata->pos;
-	  if (!display || !wtab[h] || wtab[h] == D_fore || (flayer->l_cvlist && flayer->l_cvlist->c_lnext))
-	    HelpAbort();
-#ifdef MULTIUSER
-	  else if (AclCheckPermWin(D_user, ACL_READ, wtab[h]))
-	    HelpAbort();
-#endif
-	  else
-	    ExitOverlayPage();	/* no need to redisplay */
-	  SwitchWindow(h);
-	  break;
-	case 0033:
-	case 0007:
-	  h = wlistdata->start;
-	  HelpAbort();
-	  if (h >= 0 && wtab[h])
-	    SwitchWindow(h);
-	  else if (h == -2)
-	    {
-	      struct win *p = FindNiceWindow(display ? D_other : (struct win *)0, 0);
-	      if (p)
-		SwitchWindow(p->w_number);
-	    }
-	  done = 1;
-	  break;
-	default:
-	  break;
-	}
-      ++*ppbuf;
-      --*plen;
-    }
-}
-
-static void
-WListLine(y, i, pos, isblank)
-int y, i;
-int pos;
-int isblank;
-{
-  char *str;
-  int n;
-
-  display = 0;
-  str = MakeWinMsgEv(wliststr, wtab[i], '%', flayer->l_width, (struct event *)0, 0);
-  n = strlen(str);
-  if (i != pos && isblank)
-    while (n && str[n - 1] == ' ')
-      n--;
-  LPutWinMsg(flayer, str, (i == pos || !isblank) ? flayer->l_width : n, i == pos ? &mchar_so : &mchar_blank, 0, y + 2);
-#if 0
-  LPutStr(flayer, str, n, i == pos ? &mchar_so : &mchar_blank, 0, y + 2);
-  if (i == pos || !isblank)
-    while(n < flayer->l_width)
-      LPutChar(flayer, i == pos ? &mchar_so : &mchar_blank, n++, y + 2);
-#endif
-  return;
-}
-
-static void
-WListLines(up, oldpos)
-int up, oldpos;
-{
-  struct wlistdata *wlistdata;
-  int ypos, pos;
-  int y, i, first;
-
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  ypos = wlistdata->ypos;
-  pos = wlistdata->pos;
-
-  first = ypos;
-  for (i = pos; i >= 0; i--)
-    if (wtab[i] && first-- == 0)
-      break;
-  for (y = 0; y < wlistdata->numwin; y++)
-    {
-      while (i < MAXWIN && wtab[i] == 0)
-	i++;
-      if (i == MAXWIN)
-	continue;
-      if (y == 0)
-	wlistdata->first = i;
-      wlistdata->last = i;
-      if (((i == oldpos || i == pos) && pos != oldpos) || (up > 0 && y >= wlistdata->numwin - up) || (up < 0 && y < -up))
-	WListLine(y, i, pos, i != oldpos);
-      if (i == pos)
-	wlistdata->ypos = y;
-      i++;
-    }
-}
-
-static int
-WListNormalize()
-{
-  struct wlistdata *wlistdata;
-  int i, n;
-  int ypos, pos;
-
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  ypos = wlistdata->ypos;
-  pos = wlistdata->pos;
-  if (ypos < 0)
-    ypos = 0;
-  if (ypos >= wlistdata->numwin)
-    ypos = wlistdata->numwin - 1;
-  for (n = 0, i = pos; i < MAXWIN && n < wlistdata->numwin; i++)
-    if (wtab[i])
-      n++;
-  if (ypos < wlistdata->numwin - n)
-    ypos = wlistdata->numwin - n;
-  for (n = i = 0; i < pos; i++)
-    if (wtab[i])
-      n++;
-  if (ypos > n)
-    ypos = n;
-  wlistdata->ypos = ypos;
-  wlistdata->npos = n;
-  return ypos;
-}
-
-static void
-WListMove(num, ypos)
-int num;
-int ypos;
-{
-  struct wlistdata *wlistdata;
-  int oldpos, oldypos, oldnpos;
-  int pos, up, i;
-
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  oldpos = wlistdata->pos;
-  oldypos = wlistdata->ypos;
-  oldnpos = wlistdata->npos;
-  wlistdata->ypos = ypos == -1 ? oldypos + num : ypos;
-  pos = oldpos;
-  i = pos;
-  while (num > 0 && i < MAXWIN - 1)
-    if (wtab[++i])
-      {
-	pos = i;
-	num--;
-      }
-  while (num < 0 && i > 0)
-    if (wtab[--i])
-      {
-	pos = i;
-	num++;
-      }
-  wlistdata->pos = pos;
-  ypos = WListNormalize();
-  up = wlistdata->npos - ypos - (oldnpos - oldypos);
-  if (up)
-    {
-      LScrollV(flayer, up, 2, 2 + wlistdata->numwin - 1, 0);
-      WListLines(up, oldpos);
-      LaySetCursor();
-      return;
-    }
-  if (pos == oldpos)
-    return;
-  WListLine(oldypos, oldpos, pos, 0);
-  WListLine(ypos, pos, pos, 1);
-  LaySetCursor();
-}
-
-static void
-WListRedisplayLine(y, xs, xe, isblank)
-int y, xs, xe, isblank;
-{
-  ASSERT(flayer);
-  if (y < 0)
-    {
-      wlistpage();
-      return;
-    }
-  if (y != 0 && y != flayer->l_height - 1)
-    return;
-  if (!isblank)
-    LClearArea(flayer, xs, y, xe, y, 0, 0);
-}
-
-void
-display_wlist(onblank)
-int onblank;
-{
-  struct win *p;
-  struct wlistdata *wlistdata;
-
-  if (flayer->l_width < 10 || flayer->l_height < 5)
-    {
-      LMsg(0, "Window size too small for window list page");
-      return;
-    }
-  if (onblank)
-    {
-      debug3("flayer %x %d %x\n", flayer, flayer->l_width, flayer->l_height);
-      if (!display)
-	{
-	  LMsg(0, "windowlist -b: display required");
-	  return;
-	}
-      p = D_fore;
-      SetForeWindow((struct win *)0);
-      Activate(0);
-      if (flayer->l_width < 10 || flayer->l_height < 5)
-	{
-	  LMsg(0, "Window size too small for window list page");
-	  return;
-	}
-      debug3("flayer %x %d %x\n", flayer, flayer->l_width, flayer->l_height);
-    }
-  else
-    p = Layer2Window(flayer);
-  if (InitOverlayPage(sizeof(*wlistdata), &WListLf, 0))
-    return;
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  flayer->l_x = 0;
-  flayer->l_y = flayer->l_height - 1;
-  wlistdata->start = onblank && p ? p->w_number : -1;
-  wlistdata->pos = p ? p->w_number : 0;
-  wlistdata->ypos = 0;
-  wlistdata->numwin= flayer->l_height - 3;
-  wlistpage();
-}
-
-static void
-wlistpage()
-{
-  struct wlistdata *wlistdata;
-  char *str;
-  int pos;
-
-  wlistdata = (struct wlistdata *)flayer->l_data;
-
-  LClearAll(flayer, 0);
-  if (wlistdata->start >= 0 && wtab[wlistdata->start] == 0)
-    wlistdata->start = -2;
-
-  pos = wlistdata->pos;
-  if (wtab[pos] == 0)
-    {
-      /* find new position */
-      while(++pos < MAXWIN)
-	if (wtab[pos])
-	  break;
-      if (pos == MAXWIN)
-	while (--pos > 0)
-	  if (wtab[pos])
-	    break;
-    }
-  wlistdata->pos = pos;
-
-  display = 0;
-  str = MakeWinMsgEv(wlisttit, (struct win *)0, '%', flayer->l_width, (struct event *)0, 0);
-  LPutWinMsg(flayer, str, strlen(str), &mchar_blank, 0, 0);
-  WListNormalize();
-  WListLines(wlistdata->numwin, -1);
-  LaySetCursor();
-}
-
-static void
-WListUpdate(p)
-struct win *p;
-{
-  struct wlistdata *wlistdata;
-  int i, n, y;
-
-  if (p == 0)
-    {
-      wlistpage();
-      return;
-    }
-  wlistdata = (struct wlistdata *)flayer->l_data;
-  n = p->w_number;
-  if (n < wlistdata->first || n > wlistdata->last)
-    return;
-  i = wlistdata->first;
-  for (y = 0; y < wlistdata->numwin; y++)
-    {
-      while (i < MAXWIN && wtab[i] == 0)
-	i++;
-      if (i == MAXWIN)
-	return;
-      if (i == n)
-	break;
-      i++;
-    }
-  if (y == wlistdata->numwin)
-    return;
-  WListLine(y, i, wlistdata->pos, 0);
-  LaySetCursor();
-}
-
-void
-WListUpdatecv(cv, p)
-struct canvas *cv;
-struct win *p;
-{
-  if (cv->c_layer->l_layfn != &WListLf)
-    return;
-  CV_CALL(cv, WListUpdate(p));
-}
-
-int
-InWList()
-{
-  if (flayer && flayer->l_layfn == &WListLf)
-    return 1;
-  return 0;
-}
-
-
-
 /*
 **
 **    The bindkey help page
@@ -1282,10 +766,11 @@ InWList()
 */
 
 #ifdef MAPKEYS
+extern struct kmap_ext *kmap_exts;
+extern int kmap_extn;
+extern struct action dmtab[];
+extern struct action mmtab[];
 
-extern char *kmap_extras[];
-extern int kmap_extras_fl[];
-extern struct term term[];
 
 static void BindkeyProcess __P((char **, int *));
 static void BindkeyAbort __P((void));
@@ -1310,7 +795,8 @@ static struct LayFuncs BindkeyLf =
   DefClearLine,
   DefRewrite,
   DefResize,
-  DefRestore
+  DefRestore,
+  0
 };
 
 
@@ -1335,7 +821,7 @@ struct action *tab;
   bindkeydata->tab = tab;
   
   n = 0;
-  for (i = 0; i < KMAP_KEYS+KMAP_AKEYS+KMAP_EXT; i++)
+  for (i = 0; i < KMAP_KEYS+KMAP_AKEYS+kmap_extn; i++)
     {
       if (tab[i].nr != RC_ILLEGAL)
 	n++;
@@ -1361,8 +847,9 @@ static void
 bindkeypage()
 {
   struct bindkeydata *bindkeydata;
+  struct kmap_ext *kme;
   char tbuf[256];
-  int del, i, ch, y;
+  int del, i, y, sl;
   struct action *act;
   char *xch, *s, *p;
 
@@ -1373,33 +860,43 @@ bindkeypage()
   sprintf(tbuf, "%s key bindings, page %d of %d.", bindkeydata->title, bindkeydata->page, bindkeydata->pages);
   centerline(tbuf, 0);
   y = 2;
-  for (i = bindkeydata->pos; i < KMAP_KEYS+KMAP_AKEYS+KMAP_EXT && y < flayer->l_height - 3; i++)
+  for (i = bindkeydata->pos; i < KMAP_KEYS+KMAP_AKEYS+kmap_extn && y < flayer->l_height - 3; i++)
     {
       p = tbuf;
-      act = &bindkeydata->tab[i];
-      if (act->nr == RC_ILLEGAL)
-	continue;
       xch = "   ";
       if (i < KMAP_KEYS)
 	{
+	  act = &bindkeydata->tab[i];
+	  if (act->nr == RC_ILLEGAL)
+	    continue;
 	  del = *p++ = ':';
 	  s = term[i + T_CAPS].tcname;
+	  sl = s ? strlen(s) : 0;
 	}
       else if (i < KMAP_KEYS+KMAP_AKEYS)
 	{
+	  act = &bindkeydata->tab[i];
+	  if (act->nr == RC_ILLEGAL)
+	    continue;
 	  del = *p++ = ':';
 	  s = term[i + (T_CAPS - T_OCAPS + T_CURSOR)].tcname;
+	  sl = s ? strlen(s) : 0;
 	  xch = "[A]";
 	}
       else
 	{
+	  kme = kmap_exts + (i - (KMAP_KEYS+KMAP_AKEYS));
 	  del = 0;
-	  s = kmap_extras[i - (KMAP_KEYS+KMAP_AKEYS)];
-	  if (kmap_extras_fl[i - (KMAP_KEYS+KMAP_AKEYS)])
+	  s = kme->str;
+	  sl = kme->fl & ~KMAP_NOTIMEOUT;
+	  if ((kme->fl & KMAP_NOTIMEOUT) != 0)
 	    xch = "[T]";
+	  act = bindkeydata->tab == dmtab ? &kme->dm : bindkeydata->tab == mmtab ? &kme->mm : &kme->um;
+	  if (act->nr == RC_ILLEGAL)
+	    continue;
 	}
-      while ((ch = *(unsigned char *)s++))
-	p += AddXChar(p, ch);
+      while (sl-- > 0)
+	p += AddXChar(p, *(unsigned char *)s++);
       if (del)
 	*p++ = del;
       *p++ = ' ';
@@ -1477,6 +974,61 @@ int y, xs, xe, isblank;
 #endif /* MAPKEYS */
 
 
+/*
+**
+**    The zmodem active page
+**
+*/
+
+#ifdef ZMODEM
+
+static void ZmodemRedisplayLine __P((int, int, int, int));
+static int  ZmodemResize __P((int, int));
+
+static struct LayFuncs ZmodemLf =
+{
+  DefProcess,
+  0,
+  ZmodemRedisplayLine,
+  DefClearLine,
+  DefRewrite,
+  ZmodemResize,
+  DefRestore,
+  0
+};
+
+/*ARGSUSED*/
+static int
+ZmodemResize(wi, he)
+int wi, he; 
+{
+  flayer->l_width = wi;
+  flayer->l_height = he;
+  flayer->l_x = flayer->l_width > 32 ? 32 : 0;
+  return 0;
+}
+
+static void
+ZmodemRedisplayLine(y, xs, xe, isblank)
+int y, xs, xe, isblank;
+{
+  DefRedisplayLine(y, xs, xe, isblank);
+  if (y == 0 && xs == 0)
+    LPutStr(flayer, "Zmodem active on another display", flayer->l_width > 32 ? 32 : flayer->l_width, &mchar_blank, 0, 0);
+}
+
+void
+ZmodemPage()
+{
+  if (InitOverlayPage(1, &ZmodemLf, 1))
+    return;
+  LRefreshAll(flayer, 0);
+  flayer->l_x = flayer->l_width > 32 ? 32 : 0;
+  flayer->l_y = 0;
+}
+
+#endif
+
 
 
 static void
@@ -1491,6 +1043,5 @@ int n, x, y;
     l = n;
   LPutStr(flayer, str, l, &mchar_blank, x, y);
   if (l < n)
-    LPutStr(flayer, blank, n - l, &mchar_blank, x + l, y);
+    LPutStr(flayer, (char *)blank, n - l, &mchar_blank, x + l, y);
 }
-
