@@ -1,11 +1,19 @@
-/* Copyright (c) 1993-2002
+/* Copyright (c) 2010
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 2008, 2009
+ *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
+ *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
+ *      Micah Cowan (micah@cowan.name)
+ *      Sadrul Habib Chowdhury (sadrul@users.sourceforge.net)
+ * Copyright (c) 1993-2002, 2003, 2005, 2006, 2007
  *      Juergen Weigert (jnweiger@immd4.informatik.uni-erlangen.de)
  *      Michael Schroeder (mlschroe@immd4.informatik.uni-erlangen.de)
  * Copyright (c) 1987 Oliver Laumann
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,14 +22,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * along with this program (see the file COPYING); if not, see
+ * http://www.gnu.org/licenses/, or contact Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  ****************************************************************
- * $Id: window.h,v 1.11 1994/05/31 12:33:27 mlschroe Exp $ FAU
+ * $Id$ GNU
  */
 
+#ifndef SCREEN_WINDOW_H
+#define SCREEN_WINDOW_H
 
 /* keep this in sync with the initialisations in window.c */
 struct NewWindow
@@ -123,7 +133,7 @@ struct paster
 struct paster;
 #endif
 
-struct win 
+struct win
 {
   struct win *w_next;		/* next window */
   int    w_type;		/* type of window */
@@ -135,6 +145,7 @@ struct win
   struct pseudowin *w_pwin;	/* ptr to pseudo */
 #endif
   struct display *w_pdisplay;	/* display for printer relay */
+  struct display *w_lastdisp;	/* where the last input was made */
   int	 w_number;		/* window number */
   struct event w_readev;
   struct event w_writeev;
@@ -149,6 +160,7 @@ struct win
   char  *w_akachange;		/* autoaka hack */
   char	 w_akabuf[MAXSTR];	/* aka buffer */
   int	 w_autoaka;		/* autoaka hack */
+  struct win *w_group;		/* window group we belong to */
   int	 w_intermediate;	/* char used while parsing ESC-seq */
   int	 w_args[MAXARGS];	/* emulator args */
   int	 w_NumArgs;
@@ -175,15 +187,17 @@ struct win
   int	 w_CharsetR;		/* charset number GR */
   int	 w_charsets[4];		/* Font = charsets[Charset] */
 #endif
-  int	 w_ss;		
-  int	 w_saved;
-  int	 w_Saved_x, w_Saved_y;
-  struct mchar w_SavedRend;
+  int	 w_ss;
+  struct cursor {
+    int on;
+    int	 x, y;
+    struct mchar Rend;
 #ifdef FONT
-  int	 w_SavedCharset;
-  int	 w_SavedCharsetR;
-  int	 w_SavedCharsets[4];
+    int	 Charset;
+    int	 CharsetR;
+    int	 Charsets[4];
 #endif
+  } w_saved;
   int	 w_top, w_bot;		/* scrollregion */
   int	 w_wrap;		/* autowrap */
   int	 w_origin;		/* origin mode */
@@ -215,7 +229,7 @@ struct win
   int	 w_monitor;		/* monitor status */
   int	 w_silencewait;		/* wait for silencewait secs */
   int	 w_silence;		/* silence status (Lloyd Zusman) */
-  char	 w_vbwait;            
+  char	 w_vbwait;
   char	 w_norefresh;		/* dont redisplay when switching to that win */
 #ifdef RXVT_OSC
   char	 w_xtermosc[4][MAXSTR];	/* special xterm/rxvt escapes */
@@ -234,7 +248,8 @@ struct win
 #else
   int	 w_histheight;		/* always 0 */
 #endif
-  int	 w_pid;			/* process at the other end of ptyfd */	
+  int	 w_pid;			/* process at the other end of ptyfd */
+  int	 w_deadpid;		/* saved w_pid of a process that closed the ptyfd to us */
 
   char  *w_cmdargs[MAXARGS];	/* command line argument vector */
   char	*w_dir;			/* directory for chdir */
@@ -248,6 +263,10 @@ struct win
 
   char	 w_tty[MAXSTR];
 
+  int    w_zauto;
+#ifdef ZMODEM
+  struct display *w_zdisplay;
+#endif
 #ifdef BUILTIN_TELNET
   struct sockaddr_in w_telsa;
   char   w_telbuf[IOSIZE];
@@ -259,14 +278,26 @@ struct win
   int    w_telsubidx;
   struct event w_telconnev;
 #endif
-  struct mline *w_alt_mlines;
-  int    w_alt_width;
-  int    w_alt_height;
-  int    w_alt_histheight;
-  int    w_alt_x, w_alt_y;
+  struct {
+    int    on;    /* Is the alternate buffer currently being used? */
+    struct mline *mlines;
+    int    width;
+    int    height;
 #ifdef COPY_PASTE
-  struct mline *w_alt_hlines;
-  int    w_alt_histidx;
+    int    histheight;
+    struct mline *hlines;
+    int    histidx;
+#else
+    int histheight;	/* 0 */
+#endif
+    struct cursor cursor;
+  } w_alt;
+
+  struct event w_destroyev;	/* window destroy event */
+#ifdef BSDWAIT
+  union wait w_exitstatus;	/* window exit status */
+#else
+  int w_exitstatus;
 #endif
 };
 
@@ -281,6 +312,7 @@ struct win
 #define W_TYPE_PTY		0
 #define W_TYPE_PLAIN		1
 #define W_TYPE_TELNET		2
+#define W_TYPE_GROUP		3
 
 
 /*
@@ -310,3 +342,8 @@ struct win
     : &fore->w_mlines[y - fore->w_histheight])
 
 #define Layer2Window(l) ((struct win *)(l)->l_bottom->l_data)
+
+int WindowChangeNumber __P((struct win *, int));
+
+#endif /* SCREEN_WINDOW_H */
+
